using Jypeli;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Tässä tehtävässä harjoitellaan toiston poistoa.
/// </summary>
public class T6_ToistonPoisto : PhysicsGame
{
    /// <summary>
    /// Pelikentän leveys
    /// </summary>
    const double KENTANLEVEYS = 1920;

    /// <summary>
    /// Pelikentän korkeus
    /// </summary>
    const double KENTANKORKEUS = 1080;

    /// <summary>
    /// Pelivälineiden luonti
    /// </summary>
    public override void Begin()
    {
        Level.Size = new Vector(KENTANLEVEYS, KENTANKORKEUS);
        
        Level.Background.Image = Image.CreateStarSky((int)KENTANLEVEYS, (int)KENTANKORKEUS, 100);
        SetWindowSize((int)KENTANLEVEYS, (int)KENTANKORKEUS, false);
        CenterWindow();
        Level.CreateBorders();

        PhysicsObject pelaaja1 = new PhysicsObject(80, 80, Shape.Circle)
        {
            Image = LoadImage("aajii"),
            Mass = 1,
            LinearDamping = 0.9, // Hidastaa pelaajan liikettä kiihdyttämisen jälkeen
            AngularDamping = 0.95
        };
        Add(pelaaja1);
        
        PhysicsObject pelaaja2 = new PhysicsObject(80, 80, Shape.Circle)
        {
            Image = LoadImage("troll"),
            Mass = 1,
            Position = new Vector(-100, -100),
            LinearDamping = 0.9,
            AngularDamping = 0.95
        };
        Add(pelaaja2);


        Keyboard.Listen(Key.Left, ButtonState.Down, Kaanna, "Käänny vasemmalle", pelaaja1, 2.0);
        Keyboard.Listen(Key.Right, ButtonState.Down, Kaanna, "Käänny oikealle", pelaaja1, -2.0);
        Keyboard.Listen(Key.Up, ButtonState.Down, Kiihdyta, "Kiihdytä pelaajaa 1", pelaaja1);
        Keyboard.Listen(Key.A, ButtonState.Down, Kaanna, "Käänny vasemmalle", pelaaja2, 2.0);
        Keyboard.Listen(Key.D, ButtonState.Down, Kaanna, "Käänny oikealle", pelaaja2, -2.0);
        Keyboard.Listen(Key.W, ButtonState.Down, Kiihdyta, "Kiihdytä pelaajaa 2", pelaaja2);
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, null);
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }

    /// <summary>
    /// Kääntää fysiikkaoliota 
    /// </summary>
    /// <param name="kaanneltava">Käännettävä olio</param>
    /// <param name="suuntaAsteina">Mihin käännetään (asteluku)</param>
    void Kaanna(PhysicsObject kaanneltava, double suuntaAsteina)
    {
        kaanneltava.Angle += Angle.FromDegrees(suuntaAsteina);
    }


    /// <summary>
    /// Kiihdyttää
    /// </summary>
    /// <param name="kiihdytettava">Ketä kiihdytetään</param>
    void Kiihdyta(PhysicsObject kiihdytettava)
    {
        // Vector.FromLengthAndAngle-metodi luo
        // vektorin annetusta pituudesta ja kulmasta
        // Näin saadaan pelaaja kulkemaan aina "eteenpäin"
        kiihdytettava.Push(Vector.FromLengthAndAngle(1000, kiihdytettava.Angle));
    }
}
